@@ -3,6 +3,8 @@ package Apache2::ASP::Base;
 
 use strict;
 use warnings 'all';
+use CGI ();
+use Devel::StackTrace;
 
 use Apache2::ASP::PageHandler;
 use Apache2::ASP::Request;
@@ -17,8 +19,8 @@ sub new
   my ($class, $config) = @_;
   
   my $s = bless {
-    config  => $config,
-  }, ( ref($class) || $class );
+    config  => $config, #Apache2::ASP::Config->new(),
+  }, ref($class) ? ref($class) : $class;
   
   return $s;
 }# end new()
@@ -30,134 +32,63 @@ sub setup_request
   my ($s, $r, $q) = @_;
   # Init self:
   $s->{r}           = $r;
-  $s->{'q'}         = $q;
+  $s->{q}           = $q ? $q : CGI->new();
   $s->{request}     = Apache2::ASP::Request->new( $s );
   $s->{response}    = Apache2::ASP::Response->new( $s );
   $s->{server}      = Apache2::ASP::Server->new( $s );
 
-  $s->{application} ||= $s->{config}->application_state->manager->new( $s );
-  $s->{session}     ||= $s->{config}->session_state->manager->new( $s );
+  $s->{application} = $s->{config}->application_state->manager->new( $s );
+  $s->{session}     = $s->{config}->session_state->manager->new( $s );
   $s->{global_asa}  = $s->_global_asa_class->new( $s );
   
+  # Init objects:
+  
   # Who's going to handle this request?
   my $handler = $s->resolve_request_handler( $s->r->uri );
   $s->{handler} = $handler;
   
-  return 1;
-}# end setup_request()
-
-
-#==============================================================================
-sub execute
-{
-  my ($s, $is_subrequest, @args) = @_;
-
-  if( ! $is_subrequest )
-  {
-    # Prevent multiple *OnStart events from being raised during the same request:
-    $s->global_asa->Server_OnStart()
-      unless $s->application->{"__started_server_$$"}++;
-    $s->application->save;
-    $s->global_asa->Session_OnStart()
-      unless $s->session->{__did_init}++;
-    $s->session->save;
+  # Now execute the request handler:
+  my $subref = sub {
+    my ($is_subrequest, @args) = @_;
     
-    my $filter_response = -1;
-    foreach my $filter ( $s->resolve_request_filters( $ENV{REQUEST_URI} ) )
+    if( ! $is_subrequest )
     {
-      last if $filter_response != -1;
-      $filter->init_asp_objects( $s );
-      $filter_response = eval { $s->run_filter( $filter ) };
-      if( $@ )
-      {
-        $s->global_asa->can('Script_OnError')->( $@ );
-        $s->response->{ApacheStatus} = 500;
-        $s->response->Flush;
-        return 500;
-      }# end if()
-      if( $filter_response != -1 )
-      {
-        $s->response->{ApacheStatus} = $filter_response;
-        $s->response->Flush;
-        return $filter_response;
-      }# end if()
-    }# end foreach()
+      # Prevent multiple *OnStart events from being raised during the same request:
+      $s->global_asa->can('Application_OnStart')->() 
+        unless $s->application->{__did_init}++;
+      $s->application->save;
+      $s->global_asa->can('Session_OnStart')->()
+        unless $s->session->{__did_init}++;
+      $s->session->save;
+      
+      # Now that we've initialized our other objects, we can safely call Script_OnStart()
+      $s->global_asa->can('Script_OnStart')->();
+    }# end if()
     
-    # Now that we've initialized our other objects, we can safely call Script_OnStart()
-    $s->global_asa->Script_OnStart();
-  }# end if()
-  
-  $s->{handler}->init_asp_objects( $s );
-  
-  my $handler_response;
-  if( ! $s->{did_end} )
-  {
+    $handler->init_asp_objects( $s );
     eval {
-      $handler_response = $s->run_handler( $s->{handler}, @args );
+      $handler->run( $s, @args );
       $s->response->Flush;
     };
     if( $@ )
     {
-      warn $@;
-      $s->global_asa->can('Script_OnError')->( $@ );
-      unless( $is_subrequest )
-      {
-        $s->response->{ApacheStatus} = 500;
-        $s->response->Flush;
-        $s->session->{__lastArgs} = $s->request->Form;
-      }# end unless()
-      return 500;
+      $s->global_asa->can('Script_OnError')->( Devel::StackTrace->new() );
+      $s->response->Flush;
+      return $s->response->{ApacheStatus} = 500;
     }# end if()
-  }# end if()
-  
-  if( ! $is_subrequest )
-  {
-    $s->global_asa->Script_OnEnd();
     
-    # Using __lastPage instead of HTTP_REFERER prevents us from losing that data
-    # after a JavaScript-initialized request:
-    $s->session->{__lastPage} = $ENV{REQUEST_URI};
-    $s->session->save();
-    $s->application->save();
-  }# end if()
-  $s->response->Flush;
+    $s->response->Flush;
+    if( ! $is_subrequest )
+    {
+      $s->global_asa->can('Script_OnEnd')->();
+      $s->session->save();
+    }# end if()
+    
+    return $s->response->{ApacheStatus};
+  };# end sub { }
   
-  if( $s->{handler}->isa('Apache2::ASP::RequestFilter') )
-  {
-    return $handler_response;
-  }
-  else
-  {
-    return $s->response->{Status};
-  }# end if()
-}# end execute()
-
-
-#==============================================================================
-sub run_handler
-{
-  my ($s, $handler, @args) = @_;
-  
-  $handler->before_run( $s, @args );
-  my $res = $handler->run( $s, @args );
-  $handler->after_run( $s, @args );
-  return $res;
-}# end run_handler()
-
-
-#==============================================================================
-sub run_filter
-{
-  my ($s, $handler, @args) = @_;
-  
-  $handler->before_run( $s, @args );
-  my $res = $handler->run( $s, @args );
-  $handler->after_run( $s, @args );
-  
-  # Default to -1:
-  $res = -1 unless defined($res);
-  return $res;
-}# end run_filter()
+  return $subref;
+}# end setup_request()
 
 
 #==============================================================================
@@ -166,15 +97,12 @@ sub resolve_request_handler
 {
   my ($s, $uri) = @_;
   
-  no warnings 'uninitialized';
   if( $uri =~ m/^\/handlers\// )
   {
     # (Try to) load up the handler:
     my ($handler) = $uri =~ m/^\/handlers\/([^\?]+)/;
     $handler =~ s/[^a-z0-9]/\//ig;
-		(my $file = $handler . '.pm');
-    eval { require $file }
-      unless $INC{$file};
+    eval { require "$handler.pm" };
     if( $@ )
     {
       # Failed to load the handler:
@@ -182,7 +110,6 @@ sub resolve_request_handler
     }
     else
     {
-      $handler =~ s/\//::/g;
       return $handler;
     }# end if()
   }
@@ -193,60 +120,10 @@ sub resolve_request_handler
 }# end _resolve_request_handler()
 
 
-#==============================================================================
-sub resolve_request_filters
-{
-  my ($s, $uri) = @_;
-  
-  # Bail out unless the config specifies any filters:
-  return unless my @filters = $s->config->request_filters;
-  
-  # Try to find some filters that match our URI:
-  my @matched = ();
-  foreach my $filter ( @filters )
-  {
-    if( $filter->{uri_equals} && ( $filter->{uri_equals} eq $uri ) )
-    {
-      push( @matched, $filter->{class} );
-    }
-    elsif( $filter->{uri_match} && ( $uri =~ m/$filter->{uri_match}/ ) )
-    {
-      push( @matched, $filter->{class} );
-    }# end if()
-  }# end foreach()
-  
-  # Require all of the filters:
-  foreach my $class ( @matched )
-  {
-    (my $file = $class . ".pm") =~ s/::/\//g;
-    require $file
-      unless $INC{$file};
-    
-    # Trouble's a-brewin'
-    unless( $class->isa('Apache2::ASP::RequestFilter') )
-    {
-      if( $class->can('run') )
-      {
-        # We'll let this one by, for now:
-        warn "Filter class '$class' is specified in config but does not inherit from 'Apache2::ASP::RequestFilter'";
-      }
-      else
-      {
-        # We just can't work under these conditions!:
-        die "Filter class '$class' is specified in config but does not inherit from 'Apache2::ASP::RequestFilter' and does not define a 'run()' method.";
-      }# end if()
-    }# end unless()
-  }# end foreach()
-  
-  # Done:
-  return @matched;
-}# end resolve_request_filters()
-
-
 #==============================================================================
 sub config      { $_[0]->{config}       }
 sub r           { $_[0]->{r}            }
-sub q           { $_[0]->{'q'}          }
+sub q           { $_[0]->{q}            }
 sub session     { $_[0]->{session}      }
 sub request     { $_[0]->{request}      }
 sub response    { $_[0]->{response}     }
@@ -262,9 +139,8 @@ sub _global_asa_class
   
   if( -f $s->config->www_root . '/GlobalASA.pm' )
   {
-    my $file = $s->config->www_root . '/GlobalASA.pm';
-    require $file unless $INC{$file};
-    return $s->config->application_name . '::GlobalASA';
+    require $s->config->www_root . '/GlobalASA.pm';
+    return 'GlobalASA';
   }
   else
   {
@@ -272,14 +148,6 @@ sub _global_asa_class
   }# endif()
 }# end _global_asa_class()
 
-
-#==============================================================================
-sub DESTROY
-{
-  my $s = shift;
-  delete($s->{$_}) foreach keys(%$s);
-}# end DESTROY()
-
 1;# return true:
 
 __END__
@@ -341,9 +209,10 @@ Apache2::ASP::Base - Base class for ASP engines
   {
     my ($s, $err) = @_;
     
-    war $err;
+    my $stack = Devel::StackTrace->new;
+    warn $stack->as_string;
     $s->response->Clear();
-    $s->global_asa->can('Script_OnError')->( $err );
+    $s->global_asa->can('Script_OnError')->( $stack );
     
     return 500;
   }# end _handle_error()
@@ -423,7 +292,7 @@ Use RT L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Apache2-ASP> to submit bug rep
 
 =head1 HOMEPAGE
 
-Please visit the Apache2::ASP homepage at L<http://www.devstack.com/> to see examples
+Please visit the Apache2::ASP homepage at L<http://apache2-asp.no-ip.org/> to see examples
 of Apache2::ASP in action.
 
 =head1 AUTHOR
